'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('./_virtual/_rollupPluginBabelHelpers.js');
var crypto = require('crypto');
var safeCompare = require('safe-compare');
var bodyParser = require('koa-bodyparser');
var mount = require('koa-mount');
var compose = require('koa-compose');
var network = require('@shopify/network');
var types = require('./types.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var safeCompare__default = /*#__PURE__*/_interopDefaultLegacy(safeCompare);
var bodyParser__default = /*#__PURE__*/_interopDefaultLegacy(bodyParser);
var mount__default = /*#__PURE__*/_interopDefaultLegacy(mount);
var compose__default = /*#__PURE__*/_interopDefaultLegacy(compose);

function receiveWebhook(_ref) {
  var secret = _ref.secret,
    path = _ref.path,
    _ref$onReceived = _ref.onReceived,
    onReceived = _ref$onReceived === void 0 ? noop : _ref$onReceived;
  function receiveWebhookMiddleware(_x, _x2) {
    return _receiveWebhookMiddleware.apply(this, arguments);
  }
  function _receiveWebhookMiddleware() {
    _receiveWebhookMiddleware = _rollupPluginBabelHelpers.asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(ctx, next) {
      var hmac, topic, domain, rawBody, generatedHash, graphqlTopic;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              hmac = ctx.get(types.WebhookHeader.Hmac);
              topic = ctx.get(types.WebhookHeader.Topic);
              domain = ctx.get(types.WebhookHeader.Domain);
              rawBody = ctx.request.rawBody;
              generatedHash = crypto.createHmac('sha256', secret).update(rawBody, 'utf8').digest('base64');
              graphqlTopic = topic.toUpperCase().replace(/\//g, '_');
              if (!safeCompare__default["default"](generatedHash, hmac)) {
                _context.next = 15;
                break;
              }
              ctx.res.statusCode = network.StatusCode.Accepted;
              ctx.state.webhook = {
                topic: graphqlTopic,
                domain: domain,
                payload: JSON.parse(rawBody)
              };
              _context.next = 11;
              return onReceived(ctx);
            case 11:
              _context.next = 13;
              return next();
            case 13:
              _context.next = 16;
              break;
            case 15:
              ctx.res.statusCode = network.StatusCode.Unauthorized;
            case 16:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    return _receiveWebhookMiddleware.apply(this, arguments);
  }
  var middleware = compose__default["default"]([bodyParser__default["default"](), receiveWebhookMiddleware]);
  return path ? mount__default["default"](path, middleware) : middleware;
}
function noop() {}

exports.receiveWebhook = receiveWebhook;
