'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('./_virtual/_rollupPluginBabelHelpers.js');
var network = require('@shopify/network');
var types = require('./types.js');

exports.DeliveryMethod = void 0;
(function (DeliveryMethod) {
  DeliveryMethod["Http"] = "http";
  DeliveryMethod["EventBridge"] = "eventbridge";
})(exports.DeliveryMethod || (exports.DeliveryMethod = {}));
function registerWebhook(_x) {
  return _registerWebhook.apply(this, arguments);
}
function _registerWebhook() {
  _registerWebhook = _rollupPluginBabelHelpers.asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(_ref) {
    var _headers;
    var address, topic, accessToken, shop, apiVersion, includeFields, _ref$deliveryMethod, deliveryMethod, response, result;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            address = _ref.address, topic = _ref.topic, accessToken = _ref.accessToken, shop = _ref.shop, apiVersion = _ref.apiVersion, includeFields = _ref.includeFields, _ref$deliveryMethod = _ref.deliveryMethod, deliveryMethod = _ref$deliveryMethod === void 0 ? exports.DeliveryMethod.Http : _ref$deliveryMethod;
            _context.next = 3;
            return fetch("https://".concat(shop, "/admin/api/").concat(apiVersion, "/graphql.json"), {
              method: network.Method.Post,
              body: buildQuery(topic, address, deliveryMethod, includeFields),
              headers: (_headers = {}, _rollupPluginBabelHelpers.defineProperty(_headers, types.WebhookHeader.AccessToken, accessToken), _rollupPluginBabelHelpers.defineProperty(_headers, network.Header.ContentType, 'application/graphql'), _headers)
            });
          case 3:
            response = _context.sent;
            _context.next = 6;
            return response.json();
          case 6:
            result = _context.sent;
            return _context.abrupt("return", {
              success: isSuccess(result, deliveryMethod),
              result: result
            });
          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _registerWebhook.apply(this, arguments);
}
function isSuccess(result, deliveryMethod) {
  switch (deliveryMethod) {
    case exports.DeliveryMethod.Http:
      return Boolean(result.data && result.data.webhookSubscriptionCreate && result.data.webhookSubscriptionCreate.webhookSubscription);
    case exports.DeliveryMethod.EventBridge:
      return Boolean(result.data && result.data.eventBridgeWebhookSubscriptionCreate && result.data.eventBridgeWebhookSubscriptionCreate.webhookSubscription);
  }
}
function buildArgs(args) {
  var formattedArgs = Object.entries(args).filter(function (_ref2) {
    var _ref3 = _rollupPluginBabelHelpers.slicedToArray(_ref2, 2);
      _ref3[0];
      var value = _ref3[1];
    return typeof value !== 'undefined';
  }).map(function (_ref4) {
    var _ref5 = _rollupPluginBabelHelpers.slicedToArray(_ref4, 2),
      key = _ref5[0],
      value = _ref5[1];
    return "".concat(key, ": ").concat(JSON.stringify(value));
  });
  return "{".concat(formattedArgs.join(', '), "}");
}
function buildQuery(topic, address, deliveryMethod, includeFields) {
  var mutationName;
  var webhookSubscriptionArgs;
  switch (deliveryMethod) {
    case exports.DeliveryMethod.Http:
      mutationName = 'webhookSubscriptionCreate';
      webhookSubscriptionArgs = buildArgs({
        callbackUrl: address,
        includeFields: includeFields
      });
      break;
    case exports.DeliveryMethod.EventBridge:
      mutationName = 'eventBridgeWebhookSubscriptionCreate';
      webhookSubscriptionArgs = buildArgs({
        arn: address,
        includeFields: includeFields
      });
      break;
  }
  return "\n    mutation webhookSubscriptionCreate {\n      ".concat(mutationName, "(topic: ").concat(topic, ", webhookSubscription: ").concat(webhookSubscriptionArgs, ") {\n        userErrors {\n          field\n          message\n        }\n        webhookSubscription {\n          id\n        }\n      }\n    }\n  ");
}

exports.registerWebhook = registerWebhook;
