import { asyncToGenerator as _asyncToGenerator } from './_virtual/_rollupPluginBabelHelpers.mjs';
import { createHmac } from 'crypto';
import safeCompare from 'safe-compare';
import bodyParser from 'koa-bodyparser';
import mount from 'koa-mount';
import compose from 'koa-compose';
import { StatusCode } from '@shopify/network';
import { WebhookHeader } from './types.mjs';

function receiveWebhook(_ref) {
  var secret = _ref.secret,
    path = _ref.path,
    _ref$onReceived = _ref.onReceived,
    onReceived = _ref$onReceived === void 0 ? noop : _ref$onReceived;
  function receiveWebhookMiddleware(_x, _x2) {
    return _receiveWebhookMiddleware.apply(this, arguments);
  }
  function _receiveWebhookMiddleware() {
    _receiveWebhookMiddleware = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(ctx, next) {
      var hmac, topic, domain, rawBody, generatedHash, graphqlTopic;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              hmac = ctx.get(WebhookHeader.Hmac);
              topic = ctx.get(WebhookHeader.Topic);
              domain = ctx.get(WebhookHeader.Domain);
              rawBody = ctx.request.rawBody;
              generatedHash = createHmac('sha256', secret).update(rawBody, 'utf8').digest('base64');
              graphqlTopic = topic.toUpperCase().replace(/\//g, '_');
              if (!safeCompare(generatedHash, hmac)) {
                _context.next = 15;
                break;
              }
              ctx.res.statusCode = StatusCode.Accepted;
              ctx.state.webhook = {
                topic: graphqlTopic,
                domain: domain,
                payload: JSON.parse(rawBody)
              };
              _context.next = 11;
              return onReceived(ctx);
            case 11:
              _context.next = 13;
              return next();
            case 13:
              _context.next = 16;
              break;
            case 15:
              ctx.res.statusCode = StatusCode.Unauthorized;
            case 16:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    return _receiveWebhookMiddleware.apply(this, arguments);
  }
  var middleware = compose([bodyParser(), receiveWebhookMiddleware]);
  return path ? mount(path, middleware) : middleware;
}
function noop() {}

export { receiveWebhook };
